/*
 * File:     DCRTest.java
 * Project:  MPI Linguistic Application
 * Date:     28 January 2009
 *
 * Copyright (C) 2001-2009  Max Planck Institute for Psycholinguistics
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package mpi.dcr;

import mpi.dcr.ISO12620.DataCategory;
import mpi.dcr.ISO12620.DataCategorySelection;
import mpi.dcr.ISO12620.DataCategorySummary;
import mpi.dcr.ISO12620.DataElement;
import mpi.dcr.ISO12620.Definition;
import mpi.dcr.ISO12620.DefinitionBrack;
import mpi.dcr.ISO12620.Profile;
import mpi.dcr.ISO12620.ProfileList;

import java.io.IOException;
import java.io.InputStream;

import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.JFrame;
import javax.swing.JWindow;
import javax.swing.border.TitledBorder;


/**
 * DOCUMENT ME!
 * $Id: jalopy_gnu_src_dist.xml 12249 2008-05-15 13:44:03Z hasloe $
 * @author $Author$
 * @version $Revision$
 */
public class DCRTest {
    /**
     * Creates a new DCRTest instance
     */
    public DCRTest() {
        super();
    }

    /**
     * DOCUMENT ME!
     */
    void runTest() {
        //testBare();
        String dcrLoc = "http://syntax.inist.fr/mod_webservice/call.php";
        String dcrName = "SYNTAX ISO21620";
        LATDCRConnector connector = new LATDCRConnector();
        connector.setDCRLocation(dcrLoc);
        connector.setName(dcrName);

        ArrayList dcList = new ArrayList();
        DCSmall tinyDC = null;

        try {
            ProfileList profileList = connector.getProfiles();
            Profile[] profs = profileList.getProfiles();

            for (int i = 0; i < profs.length; i++) {
                System.out.println(i + ": " + profs[i].getContent());

                //if (profs[i].getContent().equals("Terminology")) {
                if (i == 6) {
                    DataCategorySelection dcs = connector.getDataCategories(profs[i].getContent(),
                            null);
                    DataCategorySummary[] summs = dcs.getDataCategorySummaries();
                    DataCategory dc;
                    Object defObj;
                    DefinitionBrack dBrack;
                    Definition defin;

                    for (int j = 0; j < summs.length; j++) {
                        try {
                            dc = connector.getDataCategory(summs[j].getIdAsString());
                            System.out.println("\t" +
                                summs[j].getIdentifier().getContentAsString() +
                                " " + dc.getId());

                            //System.out.println("\t\t" + dc.getDescription());
                            tinyDC = new DCSmall(profs[i].getContent(),
                                    dc.getId(),
                                    summs[j].getIdentifier().getContentAsString());

                            if ((dc.getDescription() != null) &&
                                    (dc.getDescription().getDefinitions() != null)) {
                                defObj = dc.getDescription().getDefinition(0);

                                if (defObj instanceof DefinitionBrack) {
                                    dBrack = (DefinitionBrack) defObj;
                                    defin = dBrack.getDefinition();
                                } else if (defObj instanceof Definition) {
                                    defin = (Definition) defObj;
                                } else {
                                    defin = null;
                                }

                                if (defin != null) {
                                    tinyDC.setDesc(defin.getContentAsString());
                                    System.out.println("\t\t" +
                                        defin.getContentAsString());
                                } else {
                                    System.out.println("Definition is null...");
                                }
                            } else {
                                if (dc.getDescription() == null) {
                                    System.out.println("Description is null...");
                                } else {
                                    System.out.println("Definitions is null...");
                                }
                            }

                            if (dc.getDescription().getBroaderConceptGeneric() != null) {
                                System.out.println("\t\t" +
                                    dc.getDescription()
                                      .getBroaderConceptGeneric()
                                      .getContentAsString());
                                tinyDC.setBroaderDCId(dc.getDescription()
                                                        .getBroaderConceptGeneric()
                                                        .getContentAsString());
                            }

                            if (dc.getDescription().getDataElements() != null) {
                                DataElement[] elems = dc.getDescription()
                                                        .getDataElements();

                                for (int k = 0; k < elems.length; k++) {
                                    System.out.println("\t\t\t" +
                                        elems[k].getName().getContentAsString());
                                }
                            }

                            dcList.add(tinyDC);
                        } catch (DCRConnectorException dce) {
                            System.out.println(dce.getMessage());
                        }
                    }
                }
            }

            System.out.println("Number of DataCategories: " + dcList.size());
        } catch (DCRConnectorException dce) {
            System.out.println(dce.getMessage());
            dce.printStackTrace();
        }

        /*
try {
    ArrayList keywords = new ArrayList(1);
    keywords.add("noun");

    DataCategorySelection dcs = connector.searchDataCategories_2(keywords, null, "MorphoSyntax", null);
    DataCategorySummary[] summs = dcs.getDataCategorySummaries();
    DataCategory dc;

    for (int i = 0; i < summs.length; i++) {
        dc = connector.getDataCategory_2(summs[i].getIdAsString());

        System.out.println("\t" + summs[i].getIdentifier().getContentAsString() + " " + dc.getId());
        System.out.println("\t\t" + dc.getDescription());    
    }


} catch (DCRConnectorException dce) {
    System.out.println(dce.getMessage());
    dce.printStackTrace();
}
*/
    }

    /**
     * DOCUMENT ME!
     */
    void testBare() {
        HttpURLConnection conn = null;

        try {
            URL url = new URL(
                    "http://syntax.inist.fr/mod_webservice/call.php?fct=getProfiles");
            conn = (HttpURLConnection) url.openConnection();
            conn.setDefaultUseCaches(true);

            //conn.setDefaultAllowUserInteraction(true);
            conn.setRequestMethod("GET");

            //conn.setRequestProperty("fct", "getProfiles");
            conn.connect();
            System.out.println("Is proxy: " + conn.usingProxy());

            Map headers = conn.getHeaderFields();
            System.out.println("Headers num: " + headers.size());

            Iterator headIt = headers.keySet().iterator();
            Object key;

            while (headIt.hasNext()) {
                key = headIt.next();
                System.out.println(headers.get(key));
            }

            Object obj = conn.getContent();
            System.out.println("Content: " + obj);

            if (obj instanceof InputStream) {
                System.out.println("InputStream content handler");

                InputStream isObj = (InputStream) obj;
                int numBytes = 0;
                byte[] bytes = new byte[256];

                while ((numBytes = isObj.read(bytes)) > -1) {
                    System.out.println(new String(bytes, 0, numBytes));
                }

                // deze input stream kan gebruikt worden als argument voor ProfileList
            }

            System.out.println("Content length: " + conn.getContentLength());
            System.out.println("Content type: " + conn.getContentType());

            InputStream is = conn.getInputStream();
            int numBytes = 0;
            byte[] bytes = new byte[256];

            while ((numBytes = is.read(bytes)) > -1) {
                System.out.println(new String(bytes, 0, numBytes));
            }
        } catch (MalformedURLException mue) {
            mue.printStackTrace();
        } catch (IOException ioe) {
            ioe.printStackTrace();
        } finally {
            if (conn != null) {
                conn.disconnect();
            }
        }
    }

    /**
     * DOCUMENT ME!
     */
    public void uiTest() {
        String dcrLoc = "http://syntax.inist.fr/mod_webservice/call.php";
        String dcrName = "SYNTAX ISO21620";
        LATDCRConnector connector = new LATDCRConnector();
        connector.setDCRLocation(dcrLoc);
        connector.setName(dcrName);

        /*
ProfileSelectPanel pp = new ProfileSelectPanel(connector);
JFrame frame = new JFrame();
frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
frame.setContentPane(pp);
frame.pack();
frame.setVisible(true);
//pp.getSelectedProfileNames();
*/

        //RemoteDCSelectPanel dcsp = new RemoteDCSelectPanel(connector);
        LocalDCRConnector lc = new LocalDCRConnector();
        LocalDCSelectPanel dcsp = new LocalDCSelectPanel(lc);

        //dcsp.addProfiles(pp.getSelectedProfileNames());
        JFrame catFrame = new JFrame();
        catFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        catFrame.setContentPane(dcsp);
        catFrame.pack();
        catFrame.setVisible(true);
    }

    private void perfTest() {
        String dcrLoc = "http://syntax.inist.fr/mod_webservice/call.php";
        String dcrName = "SYNTAX ISO21620";
        long start = System.currentTimeMillis();

        /*
 DCRConnector conn1 = new DCRConnector();
conn1.setDCRLocation(dcrLoc);
conn1.setName(dcrName);
try {
    DataCategorySelection dcs = conn1.getDataCategories("Terminology", null);
    System.out.println("First attemp: " + (System.currentTimeMillis() - start));
} catch (Exception ee) {

}
*/
        long start2 = System.currentTimeMillis();

        LATDCRConnector connector = new LATDCRConnector();
        connector.setDCRLocation(dcrLoc);
        connector.setName(dcrName);

        try {
            DataCategorySelection dcs2 = connector.getDataCategories("Terminology",
                    null);
            System.out.println("Sec attemp: " +
                (System.currentTimeMillis() - start2));
        } catch (Exception ee2) {
        }
    }

    /**
 * @param args
 */
    public static void main(String[] args) {
        Logger.getLogger("DCR").setLevel(Level.SEVERE);

        DCRTest dt = new DCRTest();

        //dt.runTest();
        dt.uiTest();

        //dt.perfTest();
    }
}
