/*
 * File:     DataElement.java
 * Project:  MPI Linguistic Application
 * Date:     28 January 2009
 *
 * Copyright (C) 2001-2009  Max Planck Institute for Psycholinguistics
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/*
 * The Relaxer artifact
 * Copyright (c) 2000-2004, ASAMI Tomoharu, All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer. 
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package mpi.dcr.ISO12620;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.Reader;
import java.io.Writer;

import java.net.URL;

import javax.xml.parsers.ParserConfigurationException;


/**
 * NOTE: SOURCE HAS BEEN TEMPRAILY REMOVED (set to ZERO OR ONE) TO MATCH SYNTAX OUTPUT
 * <b>DataElement</b> is generated from ISO12620_revised.rng by Relaxer.
 * This class is derived from:
 * 
 * <!-- for programmer
 * <element java:className="DataElement" name="struct">
 *       <attribute name="type">
 *         <value>DE</value>
 *       </attribute>
 *       <ref name="attId"/>
 *       <ref java:propertyName="name" name="name_struct_DE"/>
 *         <zeroOrMore>      
 *             <ref java:propertyName="source" name="source_struct_DE"/>
 *       </zeroOrMore>
 *       <zeroOrMore>
 *         <ref java:propertyName="note" name="note_struct_DE"/>
 *       </zeroOrMore>
 *       <ref java:propertyName="status" name="status_struct_DE"/>
 *     </element>-->
 * <!-- for javadoc -->
 * <pre> &lt;element java:className="DataElement" name="struct"&gt;
 *       &lt;attribute name="type"&gt;
 *         &lt;value&gt;DE&lt;/value&gt;
 *       &lt;/attribute&gt;
 *       &lt;ref name="attId"/&gt;
 *       &lt;ref java:propertyName="name" name="name_struct_DE"/&gt;
 *       &lt;zeroOrMore&gt;
 *           &lt;ref java:propertyName="source" name="source_struct_DE"/&gt;
 *       &lt;/zeroOrMore&gt;
 *       &lt;zeroOrMore&gt;
 *         &lt;ref java:propertyName="note" name="note_struct_DE"/&gt;
 *       &lt;/zeroOrMore&gt;
 *       &lt;ref java:propertyName="status" name="status_struct_DE"/&gt;
 *     &lt;/element&gt;</pre>
 *
 * @version ISO12620_revised.rng (Tue Oct 04 12:49:25 CEST 2005)
 * @author  Relaxer 1.1b (http://www.relaxer.org)
 */
public class DataElement implements java.io.Serializable, Cloneable {
    /**
 * serialVersionID(generated)
 */
    private static final long serialVersionUID = -1230139052174002628L;

    /** Holds value of property DOCUMENT ME! */
    public static final String TYPE_DE = "DE";
    private String type_;
    private String id_;
    private Name name_;
    private Source source_;

    // List<Note>
    private java.util.List note_ = new java.util.ArrayList();
    private Status status_;

    /**
 * Creates a <code>DataElement</code>.
 *
 */
    public DataElement() {
        type_ = "";
    }

    /**
 * Creates a <code>DataElement</code>.
 *
 * @param source
 */
    public DataElement(DataElement source) {
        setup(source);
    }

    /**
 * Creates a <code>DataElement</code> by the Stack <code>stack</code>
 * that contains Elements.
 * This constructor is supposed to be used internally
 * by the Relaxer system.
 *
 * @param stack
 */
    public DataElement(RStack stack) {
        setup(stack);
    }

    /**
 * Creates a <code>DataElement</code> by the Document <code>doc</code>.
 *
 * @param doc
 */
    public DataElement(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
 * Creates a <code>DataElement</code> by the Element <code>element</code>.
 *
 * @param element
 */
    public DataElement(Element element) {
        setup(element);
    }

    /**
 * Creates a <code>DataElement</code> by the File <code>file</code>.
 *
 * @param file
 * @exception IOException
 * @exception SAXException
 * @exception ParserConfigurationException
 */
    public DataElement(File file)
        throws IOException, SAXException, ParserConfigurationException {
        setup(file);
    }

    /**
 * Creates a <code>DataElement</code>
 * by the String representation of URI <code>uri</code>.
 *
 * @param uri
 * @exception IOException
 * @exception SAXException
 * @exception ParserConfigurationException
 */
    public DataElement(String uri)
        throws IOException, SAXException, ParserConfigurationException {
        setup(uri);
    }

    /**
 * Creates a <code>DataElement</code> by the URL <code>url</code>.
 *
 * @param url
 * @exception IOException
 * @exception SAXException
 * @exception ParserConfigurationException
 */
    public DataElement(URL url)
        throws IOException, SAXException, ParserConfigurationException {
        setup(url);
    }

    /**
 * Creates a <code>DataElement</code> by the InputStream <code>in</code>.
 *
 * @param in
 * @exception IOException
 * @exception SAXException
 * @exception ParserConfigurationException
 */
    public DataElement(InputStream in)
        throws IOException, SAXException, ParserConfigurationException {
        setup(in);
    }

    /**
 * Creates a <code>DataElement</code> by the InputSource <code>is</code>.
 *
 * @param is
 * @exception IOException
 * @exception SAXException
 * @exception ParserConfigurationException
 */
    public DataElement(InputSource is)
        throws IOException, SAXException, ParserConfigurationException {
        setup(is);
    }

    /**
 * Creates a <code>DataElement</code> by the Reader <code>reader</code>.
 *
 * @param reader
 * @exception IOException
 * @exception SAXException
 * @exception ParserConfigurationException
 */
    public DataElement(Reader reader)
        throws IOException, SAXException, ParserConfigurationException {
        setup(reader);
    }

    /**
 * Initializes the <code>DataElement</code> by the DataElement <code>source</code>.
 *
 * @param source
 */
    public void setup(DataElement source) {
        int size;
        setType(source.getType());
        setId(source.getId());

        if (source.name_ != null) {
            setName((Name) source.getName().clone());
        }

        if (source.source_ != null) {
            setSource((Source) source.getSource().clone());
        }

        this.note_.clear();
        size = source.note_.size();

        for (int i = 0; i < size; i++) {
            addNote((Note) source.getNote(i).clone());
        }

        if (source.status_ != null) {
            setStatus((Status) source.getStatus().clone());
        }
    }

    /**
 * Initializes the <code>DataElement</code> by the Document <code>doc</code>.
 *
 * @param doc
 */
    public void setup(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
 * Initializes the <code>DataElement</code> by the Element <code>element</code>.
 *
 * @param element
 */
    public void setup(Element element) {
        init(element);
    }

    /**
 * Initializes the <code>DataElement</code> by the Stack <code>stack</code>
 * that contains Elements.
 * This constructor is supposed to be used internally
 * by the Relaxer system.
 *
 * @param stack
 */
    public void setup(RStack stack) {
        init(stack.popElement());
    }

    /**
 * @param element
 */
    private void init(Element element) {
        RStack stack = new RStack(element);
        type_ = URelaxer.getAttributePropertyAsString(element, "type");
        id_ = URelaxer.getAttributePropertyAsString(element, "id");
        setName(new Name(stack));

        //TODO: Temporarily changed to match Syntax outptu
        if (Source.isMatch(stack)) {
            setSource(new Source(stack));
        }

        //setSource(new Source(stack));
        note_.clear();

        while (true) {
            if (Note.isMatch(stack)) {
                addNote(new Note(stack));
            } else {
                break;
            }
        }

        setStatus(new Status(stack));
    }

    /**
 * @return Object
 */
    public Object clone() {
        return (new DataElement(this));
    }

    /**
 * Creates a DOM representation of the object.
 * Result is appended to the Node <code>parent</code>.
 *
 * @param parent
 */
    public void makeElement(Node parent) {
        Document doc;

        if (parent instanceof Document) {
            doc = (Document) parent;
        } else {
            doc = parent.getOwnerDocument();
        }

        Element element = doc.createElement("struct");
        int size;

        if (this.type_ != null) {
            URelaxer.setAttributePropertyByString(element, "type", this.type_);
        }

        if (this.id_ != null) {
            URelaxer.setAttributePropertyByString(element, "id", this.id_);
        }

        this.name_.makeElement(element);
        this.source_.makeElement(element);
        size = this.note_.size();

        for (int i = 0; i < size; i++) {
            Note value = (Note) this.note_.get(i);
            value.makeElement(element);
        }

        this.status_.makeElement(element);
        parent.appendChild(element);
    }

    /**
 * Initializes the <code>DataElement</code> by the File <code>file</code>.
 *
 * @param file
 * @exception IOException
 * @exception SAXException
 * @exception ParserConfigurationException
 */
    public void setup(File file)
        throws IOException, SAXException, ParserConfigurationException {
        setup(file.toURL());
    }

    /**
 * Initializes the <code>DataElement</code>
 * by the String representation of URI <code>uri</code>.
 *
 * @param uri
 * @exception IOException
 * @exception SAXException
 * @exception ParserConfigurationException
 */
    public void setup(String uri)
        throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(uri, UJAXP.FLAG_NONE));
    }

    /**
 * Initializes the <code>DataElement</code> by the URL <code>url</code>.
 *
 * @param url
 * @exception IOException
 * @exception SAXException
 * @exception ParserConfigurationException
 */
    public void setup(URL url)
        throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(url, UJAXP.FLAG_NONE));
    }

    /**
 * Initializes the <code>DataElement</code> by the InputStream <code>in</code>.
 *
 * @param in
 * @exception IOException
 * @exception SAXException
 * @exception ParserConfigurationException
 */
    public void setup(InputStream in)
        throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(in, UJAXP.FLAG_NONE));
    }

    /**
 * Initializes the <code>DataElement</code> by the InputSource <code>is</code>.
 *
 * @param is
 * @exception IOException
 * @exception SAXException
 * @exception ParserConfigurationException
 */
    public void setup(InputSource is)
        throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(is, UJAXP.FLAG_NONE));
    }

    /**
 * Initializes the <code>DataElement</code> by the Reader <code>reader</code>.
 *
 * @param reader
 * @exception IOException
 * @exception SAXException
 * @exception ParserConfigurationException
 */
    public void setup(Reader reader)
        throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(reader, UJAXP.FLAG_NONE));
    }

    /**
 * Creates a DOM document representation of the object.
 *
 * @exception ParserConfigurationException
 * @return Document
 */
    public Document makeDocument() throws ParserConfigurationException {
        Document doc = UJAXP.makeDocument();
        makeElement(doc);

        return (doc);
    }

    /**
 * Gets the String property <b>type</b>.
 *
 * @return String
 */
    public String getType() {
        return (type_);
    }

    /**
 * Sets the String property <b>type</b>.
 *
 * @param type
 */
    public void setType(String type) {
        this.type_ = type;
    }

    /**
 * Gets the String property <b>id</b>.
 *
 * @return String
 */
    public String getId() {
        return (id_);
    }

    /**
 * Sets the String property <b>id</b>.
 *
 * @param id
 */
    public void setId(String id) {
        this.id_ = id;
    }

    /**
 * Gets the Name property <b>name</b>.
 *
 * @return Name
 */
    public Name getName() {
        return (name_);
    }

    /**
 * Sets the Name property <b>name</b>.
 *
 * @param name
 */
    public void setName(Name name) {
        this.name_ = name;
    }

    /**
 * Gets the Source property <b>source</b>.
 *
 * @return Source
 */
    public Source getSource() {
        return (source_);
    }

    /**
 * Sets the Source property <b>source</b>.
 *
 * @param source
 */
    public void setSource(Source source) {
        this.source_ = source;
    }

    /**
 * Gets the Note property <b>note</b>.
 *
 * @return Note[]
 */
    public Note[] getNote() {
        Note[] array = new Note[note_.size()];

        return ((Note[]) note_.toArray(array));
    }

    /**
 * Sets the Note property <b>note</b>.
 *
 * @param note
 */
    public void setNote(Note[] note) {
        this.note_.clear();

        for (int i = 0; i < note.length; i++) {
            addNote(note[i]);
        }
    }

    /**
 * Sets the Note property <b>note</b>.
 *
 * @param note
 */
    public void setNote(Note note) {
        this.note_.clear();
        addNote(note);
    }

    /**
 * Adds the Note property <b>note</b>.
 *
 * @param note
 */
    public void addNote(Note note) {
        this.note_.add(note);
    }

    /**
 * Adds the Note property <b>note</b>.
 *
 * @param note
 */
    public void addNote(Note[] note) {
        for (int i = 0; i < note.length; i++) {
            addNote(note[i]);
        }
    }

    /**
 * Gets number of the Note property <b>note</b>.
 *
 * @return int
 */
    public int sizeNote() {
        return (note_.size());
    }

    /**
 * Gets the Note property <b>note</b> by index.
 *
 * @param index
 * @return Note
 */
    public Note getNote(int index) {
        return ((Note) note_.get(index));
    }

    /**
 * Sets the Note property <b>note</b> by index.
 *
 * @param index
 * @param note
 */
    public void setNote(int index, Note note) {
        this.note_.set(index, note);
    }

    /**
 * Adds the Note property <b>note</b> by index.
 *
 * @param index
 * @param note
 */
    public void addNote(int index, Note note) {
        this.note_.add(index, note);
    }

    /**
 * Remove the Note property <b>note</b> by index.
 *
 * @param index
 */
    public void removeNote(int index) {
        this.note_.remove(index);
    }

    /**
 * Remove the Note property <b>note</b> by object.
 *
 * @param note
 */
    public void removeNote(Note note) {
        this.note_.remove(note);
    }

    /**
 * Clear the Note property <b>note</b>.
 *
 */
    public void clearNote() {
        this.note_.clear();
    }

    /**
 * Gets the Status property <b>status</b>.
 *
 * @return Status
 */
    public Status getStatus() {
        return (status_);
    }

    /**
 * Sets the Status property <b>status</b>.
 *
 * @param status
 */
    public void setStatus(Status status) {
        this.status_ = status;
    }

    /**
 * Makes an XML text representation.
 *
 * @return String
 */
    public String makeTextDocument() {
        StringBuffer buffer = new StringBuffer();
        makeTextElement(buffer);

        return (new String(buffer));
    }

    /**
 * Makes an XML text representation.
 *
 * @param buffer
 */
    public void makeTextElement(StringBuffer buffer) {
        int size;
        buffer.append("<struct");

        if (type_ != null) {
            buffer.append(" type=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getType())));
            buffer.append("\"");
        }

        if (id_ != null) {
            buffer.append(" id=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getId())));
            buffer.append("\"");
        }

        buffer.append(">");
        name_.makeTextElement(buffer);

        if (source_ != null) {
            source_.makeTextElement(buffer);
        }

        size = this.note_.size();

        for (int i = 0; i < size; i++) {
            Note value = (Note) this.note_.get(i);
            value.makeTextElement(buffer);
        }

        status_.makeTextElement(buffer);
        buffer.append("</struct>");
    }

    /**
 * Makes an XML text representation.
 *
 * @param buffer
 * @exception IOException
 */
    public void makeTextElement(Writer buffer) throws IOException {
        int size;
        buffer.write("<struct");

        if (type_ != null) {
            buffer.write(" type=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getType())));
            buffer.write("\"");
        }

        if (id_ != null) {
            buffer.write(" id=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getId())));
            buffer.write("\"");
        }

        buffer.write(">");
        name_.makeTextElement(buffer);

        if (source_ != null) {
            source_.makeTextElement(buffer);
        }

        size = this.note_.size();

        for (int i = 0; i < size; i++) {
            Note value = (Note) this.note_.get(i);
            value.makeTextElement(buffer);
        }

        status_.makeTextElement(buffer);
        buffer.write("</struct>");
    }

    /**
 * Makes an XML text representation.
 *
 * @param buffer
 */
    public void makeTextElement(PrintWriter buffer) {
        int size;
        buffer.print("<struct");

        if (type_ != null) {
            buffer.print(" type=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getType())));
            buffer.print("\"");
        }

        if (id_ != null) {
            buffer.print(" id=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getId())));
            buffer.print("\"");
        }

        buffer.print(">");
        name_.makeTextElement(buffer);

        if (source_ != null) {
            source_.makeTextElement(buffer);
        }

        size = this.note_.size();

        for (int i = 0; i < size; i++) {
            Note value = (Note) this.note_.get(i);
            value.makeTextElement(buffer);
        }

        status_.makeTextElement(buffer);
        buffer.print("</struct>");
    }

    /**
 * Makes an XML text representation.
 *
 * @param buffer
 */
    public void makeTextAttribute(StringBuffer buffer) {
    }

    /**
 * Makes an XML text representation.
 *
 * @param buffer
 * @exception IOException
 */
    public void makeTextAttribute(Writer buffer) throws IOException {
    }

    /**
 * Makes an XML text representation.
 *
 * @param buffer
 */
    public void makeTextAttribute(PrintWriter buffer) {
    }

    /**
 * Gets the property value as String.
 *
 * @return String
 */
    public String getTypeAsString() {
        return (URelaxer.getString(getType()));
    }

    /**
 * Gets the property value as String.
 *
 * @return String
 */
    public String getIdAsString() {
        return (URelaxer.getString(getId()));
    }

    /**
 * Sets the property value by String.
 *
 * @param string
 */
    public void setTypeByString(String string) {
        setType(string);
    }

    /**
 * Sets the property value by String.
 *
 * @param string
 */
    public void setIdByString(String string) {
        setId(string);
    }

    /**
 * Returns a String representation of this object.
 * While this method informs as XML format representaion, 
 *  it's purpose is just information, not making 
 * a rigid XML documentation.
 *
 * @return String
 */
    public String toString() {
        try {
            return (makeTextDocument());
        } catch (Exception e) {
            return (super.toString());
        }
    }

    /**
 * Tests if a Element <code>element</code> is valid
 * for the <code>DataElement</code>.
 *
 * @param element
 * @return boolean
 */
    public static boolean isMatch(Element element) {
        if (!URelaxer.isTargetElement(element, "struct")) {
            return (false);
        }

        RStack target = new RStack(element);

        if (!URelaxer.hasAttributeHungry(target, "type")) {
            return (false);
        }

        if (!URelaxer.isMatchDataValuesAttr(element, "type", "token", "DE")) {
            return (false);
        }

        if (!Name.isMatchHungry(target)) {
            return (false);
        }

        if (!Source.isMatchHungry(target)) {
            //TODO: Temporarily removed to match Syntax outptu
            //return (false);
        }

        while (true) {
            if (!Note.isMatchHungry(target)) {
                break;
            }
        }

        if (!Status.isMatchHungry(target)) {
            return (false);
        }

        if (!target.isEmptyElement()) {
            return (false);
        }

        return (true);
    }

    /**
 * Tests if elements contained in a Stack <code>stack</code>
 * is valid for the <code>DataElement</code>.
 * This mehtod is supposed to be used internally
 * by the Relaxer system.
 *
 * @param stack
 * @return boolean
 */
    public static boolean isMatch(RStack stack) {
        Element element = stack.peekElement();

        if (element == null) {
            return (false);
        }

        return (isMatch(element));
    }

    /**
 * Tests if elements contained in a Stack <code>stack</code>
 * is valid for the <code>DataElement</code>.
 * This method consumes the stack contents during matching operation.
 * This mehtod is supposed to be used internally
 * by the Relaxer system.
 *
 * @param stack
 * @return boolean
 */
    public static boolean isMatchHungry(RStack stack) {
        Element element = stack.peekElement();

        if (element == null) {
            return (false);
        }

        if (isMatch(element)) {
            stack.popElement();

            return (true);
        } else {
            return (false);
        }
    }
}
