/*
 * File:     ProfileSelectPanel.java
 * Project:  MPI Linguistic Application
 * Date:     28 January 2009
 *
 * Copyright (C) 2001-2009  Max Planck Institute for Psycholinguistics
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package mpi.dcr;

import mpi.dcr.ISO12620.Profile;
import mpi.dcr.ISO12620.ProfileList;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.border.TitledBorder;


/**
 * A panel containing elements to select a profile.
 *
 * @author Han Sloetjes
 * @version 1.0
  */
public class ProfileSelectPanel extends JPanel {
    private LATDCRConnector connector = null;
    private JScrollPane scrollPane;
    private List selectedProfs;
    private JList allProfs;
    private ResourceBundle resBundle;

    /**
     * Creates a new ProfileSelectPanel instance
     */
    public ProfileSelectPanel() {
        super();
        initComponents();
    }

    /**
     * Creates a new ProfileSelectPanel instance
     *
     * @param connector the connector
     */
    public ProfileSelectPanel(IDCRConnector connector) {
        super();

        if (connector instanceof LATDCRConnector) {
            this.connector = (LATDCRConnector) connector;
        }

        initComponents();
    }

    /**
     * Creates a new ProfileSelectPanel instance
     *
     * @param connector the connector
     * @param resBundle a resource bundle containing localized texts
     */
    public ProfileSelectPanel(IDCRConnector connector, ResourceBundle resBundle) {
        super();

        if (connector instanceof LATDCRConnector) {
            this.connector = (LATDCRConnector) connector;
        }

        this.resBundle = resBundle;

        initComponents();
    }

    private void initComponents() {
        setLayout(new GridBagLayout());
        scrollPane = new JScrollPane();

        Dimension dim = new Dimension(200, 300);
        scrollPane.setMinimumSize(dim);
        scrollPane.setPreferredSize(dim);

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.anchor = GridBagConstraints.NORTHWEST;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.insets = new Insets(2, 6, 2, 6);
        add(scrollPane, gbc);

        selectedProfs = new ArrayList(8);

        initList();

        if (resBundle != null) {
            setBorder(new TitledBorder(resBundle.getString(
                        "DCR.Label.SelectProfiles")));
        } else {
            setBorder(new TitledBorder("Select Profiles"));
        }
    }

    private void initList() {
        if (connector == null) {
            connector = new LATDCRConnector();

            // temp. use hardcoded url etc.
            connector.setDCRLocation(
                "http://syntax.inist.fr/mod_webservice/call.php");
            connector.setName("SYNTAX ISO21620");
        }

        try {
            ProfileList profileList = connector.getProfiles();
            Profile[] profs = profileList.getProfiles();
            List ap = new ArrayList(16);

            for (int i = 0; i < profs.length; i++) {
                ap.add(profs[i].getContentAsString());
            }

            allProfs = new JList(ap.toArray(new String[] {  }));
            allProfs.getSelectionModel().setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        } catch (DCRConnectorException dce) {
            System.out.println(dce.getMessage());
            dce.printStackTrace();
            allProfs = new JList();
        }

        if (allProfs != null) {
            scrollPane.setViewportView(allProfs);
        }
    }

    /**
     * Returns a list of selected profiles.
     *
     * @return a list of names of selected profiles
     */
    public List getSelectedProfileNames() {
        selectedProfs.clear();

        Object[] selObj = allProfs.getSelectedValues();

        for (int i = 0; i < selObj.length; i++) {
            selectedProfs.add(selObj[i]);
        }

        return selectedProfs;
    }
}
