/*
 * File:     ChangeCVEntryCommand.java
 * Project:  MPI Linguistic Application
 * Date:     28 January 2009
 *
 * Copyright (C) 2001-2009  Max Planck Institute for Psycholinguistics
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package mpi.eudico.client.annotator.commands;

import mpi.util.CVEntry;
import mpi.util.ControlledVocabulary;


/**
 * A Command to change an entry in a Controlled Vocabulary.
 *
 * @author Han Sloetjes
 */
public class ChangeCVEntryCommand implements Command {
    private String commandName;

    /**
     * Creates a new ChangeCVEntryCommand instance
     *
     * @param name the name of the command
     */
    public ChangeCVEntryCommand(String name) {
        commandName = name;
    }

    /**
     * <b>Note: </b>it is assumed the types and order of the arguments are
     * correct.<br>
     * When the CV is connected to a Transcription it will handle the notification
     * of the change.
     *
     * @param receiver the Controlled Vocabulary
     * @param arguments the arguments: <ul><li>arg[0] = the old value
     *        of the entry (String)</li> <li></li>arg[1] = the new value
     *        of the entry (String)<li>arg[2] = the description of
     *        the entry</li> </ul>
     */
    public void execute(Object receiver, Object[] arguments) {
        ControlledVocabulary conVoc = (ControlledVocabulary) receiver;
        String oldValue = (String) arguments[0];
        String newValue = (String) arguments[1];
        String desc = (String) arguments[2];

        CVEntry curEntry = conVoc.getEntryWithValue(oldValue);

        if (curEntry != null) {
            conVoc.modifyEntryValue(curEntry, newValue);
            curEntry.setDescription(desc);
        }
    }

    /**
     * Returns the name of the command.
     *
     * @return the name of the command
     */
    public String getName() {
        return commandName;
    }
}
