/*
 * File:     ReplaceCVEntriesCommand.java
 * Project:  MPI Linguistic Application
 * Date:     28 January 2009
 *
 * Copyright (C) 2001-2009  Max Planck Institute for Psycholinguistics
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package mpi.eudico.client.annotator.commands;

import mpi.util.CVEntry;
import mpi.util.ControlledVocabulary;


/**
 * A Command to replace the existing CVEntries in a Controlled Vocabulary  by a
 * new array of CVEntries.
 *
 * @author Han Sloetjes
 */
public class ReplaceCVEntriesCommand implements Command {
    private String commandName;
    private CVEntry[] entries;
    private ControlledVocabulary conVoc;

    /**
     * Creates a new ReplaceCVEntriesCommand instance.
     *
     * @param name the name of the command
     */
    public ReplaceCVEntriesCommand(String name) {
        commandName = name;
    }

    /**
     * <b>Note: </b>it is assumed the types and order of the arguments are  correct.<br>
     * When the CV is connected to a Transcription it will handle the
     * notification  of the change.
     *
     * @param receiver the Controlled Vocabulary
     * @param arguments the arguments: <ul><li>arg[0] = an array of entries to
     *        replace the exisitng entries (CVEntry[])</li> </ul>
     */
    public void execute(Object receiver, Object[] arguments) {
        conVoc = (ControlledVocabulary) receiver;

        entries = (CVEntry[]) arguments[0];

        if ((conVoc != null) && (entries != null)) {
            conVoc.replaceAll(entries);
        }
    }

    /**
     * Returns the name of the command.
     *
     * @return the name of the command
     */
    public String getName() {
        return commandName;
    }
}
