/*
 * File:     SegmentsToTiersDlgCommand.java
 * Project:  MPI Linguistic Application
 * Date:     28 January 2009
 *
 * Copyright (C) 2001-2009  Max Planck Institute for Psycholinguistics
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package mpi.eudico.client.annotator.commands;

import mpi.eudico.client.annotator.gui.SegmentsToTiersDialog;

import mpi.eudico.server.corpora.clomimpl.abstr.TranscriptionImpl;

import java.util.List;


/**
 * Creates a dialog to configure options for the recognizer segmentation to
 * tiers and annotations process.
 *
 * @author Han Sloetjes
 * @version 1.0
 */
public class SegmentsToTiersDlgCommand implements Command {
    private String commandName;

    /**
     * Creates a new SegmentsToTiersDlgCommand instance
     *
     * @param name name of the command
     */
    public SegmentsToTiersDlgCommand(String name) {
        commandName = name;
    }

    /**
     * <b>Note: </b>it is assumed the types and order of the arguments are
     * correct.<br>
     *
     * @param receiver the TranscriptionImpl
     * @param arguments the arguments: <ul><li>arg[0] = the list of
     *        segmentation  objects (ArrayList)</li> </ul>
     */
    public void execute(Object receiver, Object[] arguments) {
        if (receiver instanceof TranscriptionImpl && (arguments != null) &&
                arguments[0] instanceof List) {
            new SegmentsToTiersDialog(ELANCommandFactory.getRootFrame(
                    (TranscriptionImpl) receiver),
                (TranscriptionImpl) receiver, (List) arguments[0]).setVisible(true);
        }
    }

    /**
     * @see mpi.eudico.client.annotator.commands.Command#getName()
     */
    public String getName() {
        return commandName;
    }
}
