/*
 * File:     SetLocaleCA.java
 * Project:  MPI Linguistic Application
 * Date:     28 January 2009
 *
 * Copyright (C) 2001-2009  Max Planck Institute for Psycholinguistics
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package mpi.eudico.client.annotator.commands;

import mpi.eudico.client.annotator.Constants;
import mpi.eudico.client.annotator.ViewerManager2;

import java.io.File;

import javax.swing.Action;


/**
 * A CommandAction to change the application's Locale.
 *
 * @author Han Sloetjes
 */
public class SetLocaleCA extends CommandAction {
    /**
     * Creates a new SetLocaleCA instance
     *
     * @param viewerManager DOCUMENT ME!
     * @param name DOCUMENT ME!
     */
    public SetLocaleCA(ViewerManager2 viewerManager, String name) {
        super(viewerManager, name);
        putValue(Action.DEFAULT, name);
        updateLocale(); //because of mnemonic

        if (name.equals(ELANCommandFactory.CUSTOM_LANG)) {
            // check if the action should be enabled or disabled
            try {
                File custFile = new File(Constants.ELAN_DATA_DIR +
                        Constants.FILESEPARATOR + "ElanLanguage.properties");

                if (!custFile.exists()) {
                    setEnabled(false);
                }
            } catch (Exception ex) {
                // catch any exception
            }
        }
    }

    /**
     * DOCUMENT ME!
     */
    protected void newCommand() {
        command = ELANCommandFactory.createCommand(vm.getTranscription(),
                ELANCommandFactory.SET_LOCALE);
    }

    /**
     * The receiver can be null; the Command calls a static method in
     * ElanLocale
     *
     * @return DOCUMENT ME!
     */
    protected Object getReceiver() {
        return null;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    protected Object[] getArguments() {
        return new Object[] {
            ELANCommandFactory.getLocaleForKey(getValue(Action.DEFAULT))
        };
    }
}
