/*
 * File:     HelpWindow.java
 * Project:  MPI Linguistic Application
 * Date:     28 January 2009
 *
 * Copyright (C) 2001-2009  Max Planck Institute for Psycholinguistics
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package mpi.eudico.client.annotator.help;

import java.awt.Frame;
import java.awt.Window;

import java.net.URL;

import javax.help.HelpSet;
import javax.help.HelpSetException;
import javax.help.MainWindow;
import javax.help.Presentation;
import javax.help.WindowPresentation;

import javax.swing.ImageIcon;


/**
 * A singleton class that creates and returns the JavaHelp window.
 *
 * @author Han Sloetjes
 * @version 1.0
 */
public class HelpWindow {
    private static Window helpWindow = null;

    /**
     * Creates a new HelpWindow instance
     */
    private HelpWindow() {
    }

    /**
     * Creates (if necessary) and returns the single help window.
     *
     * @return the help window
     *
     * @throws HelpException if the helpfiles can not be found or the helpset
     *         can not be read, or the JavaHelp libraries could not be loaded
     */
    public static Window getHelpWindow() throws HelpException {
        if (helpWindow == null) {
            try {
                URL url = null;
                url = HelpWindow.class.getResource("/help/jhelpset.hs");

                //System.out.println("Help url: " + url);
                HelpSet hs = null;
                ClassLoader cl = HelpWindow.class.getClassLoader();
                hs = new HelpSet(cl, url);

                Presentation pres = MainWindow.getPresentation(hs, "ELAN Help");

                if (pres instanceof WindowPresentation) {
                    ((WindowPresentation) pres).createHelpWindow();
                    helpWindow = ((WindowPresentation) pres).getHelpWindow();
                }

                if (helpWindow == null) {
                    throw new HelpException("Help window is null");
                } else {
                    try {
                        ImageIcon icon = new ImageIcon(HelpWindow.class.getResource(
                                    "/mpi/eudico/client/annotator/resources/ELAN16.png"));

                        if (helpWindow instanceof Frame && (icon != null)) {
                            ((Frame) helpWindow).setIconImage(icon.getImage());
                        }
                    } catch (Exception ee) {
                    }
                }
            } catch (HelpSetException hse) {
                //hse.printStackTrace();
                throw new HelpException("Could not create help window: " +
                    hse.getMessage(), hse.getCause());
            } catch (Exception ex) {
                //ex.printStackTrace();
                throw new HelpException("Could not create help window: " +
                    ex.getMessage(), ex.getCause());
            } catch (Error err) {
                //err.printStackTrace();
                throw new HelpException("Could not create help window: " +
                    err.getMessage(), err.getCause());
            }
        }

        return helpWindow;
    }
}
