/*
 * File:     DefaultMDViewerComponent.java
 * Project:  MPI Linguistic Application
 * Date:     28 January 2009
 *
 * Copyright (C) 2001-2009  Max Planck Institute for Psycholinguistics
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package mpi.eudico.client.annotator.md;

import mpi.eudico.client.annotator.ElanLocale;

import mpi.eudico.client.annotator.md.spi.MDServiceProvider;
import mpi.eudico.client.annotator.md.spi.MDViewerComponent;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.MenuItem;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import java.util.Iterator;
import java.util.Map;

import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingUtilities;
import javax.swing.table.DefaultTableModel;


/**
 * A default panel for visualisation of selected metadata values.
 *
 * @author Han Sloetjes
 * @version 1.0
  */
public class DefaultMDViewerComponent extends JPanel
    implements MDViewerComponent {
    /** The service provider */
    protected MDServiceProvider provider;

    /** Holds value of property DOCUMENT ME! */
    protected JTable mdTable;

    /** Holds value of property DOCUMENT ME! */
    protected DefaultTableModel model;

    /** Holds value of property DOCUMENT ME! */
    protected String keyColumn = ElanLocale.getString("MetadataViewer.Key");

    /** Holds value of property DOCUMENT ME! */
    protected String valColumn = ElanLocale.getString("MetadataViewer.Value");

    /**
     * Creates a new DefaultMDViewerPanel instance
     */
    public DefaultMDViewerComponent() {
        super();

        initComponents();
    }

    /**
     * Creates a new DefaultMDViewerPanel instance
     *
     * @param provider the MD service provider
     */
    public DefaultMDViewerComponent(MDServiceProvider provider) {
        setProvider(provider);

        initComponents();
    }

    /**
     * DOCUMENT ME!
     */
    protected void initComponents() {
        model = new DefaultTableModel(0, 2);
        model.setColumnIdentifiers(new String[] { keyColumn, valColumn });
        mdTable = new JTable(model);

        JScrollPane jsp = new JScrollPane(mdTable);
        mdTable.setEnabled(false);

        setLayout(new GridBagLayout());

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.fill = GridBagConstraints.BOTH;
        gbc.weighty = 1.0;
        gbc.weightx = 1.0;
        add(jsp, gbc);
    }

    /**
     * Sets the metadata provider.
     *
     * @param provider the metadata provider
     */
    public void setProvider(MDServiceProvider provider) {
        this.provider = provider;
    }

    /**
     * Sets the selected key-value pairs for the viewer.
     *
     * @param keysAndValuesMap the map of keys and values for visualisation.
     */
    public void setSelectedKeysAndValues(Map keysAndValuesMap) {
        if (keysAndValuesMap == null) {
            return;
        }

        int rowCount = model.getRowCount();

        for (int i = rowCount - 1; i >= 0; i--) {
            model.removeRow(i);
        }

        Iterator keyIt = keysAndValuesMap.keySet().iterator();
        Object key;
        Object val;

        while (keyIt.hasNext()) {
            key = keyIt.next();
            val = keysAndValuesMap.get(key);

            //System.out.println("K: " + key + "\n\tV: " + val);
            model.addRow(new Object[] { key, val });
        }
    }

    /*
        public void mousePressed(MouseEvent e) {
            Point pp = e.getPoint();

            if ((SwingUtilities.isRightMouseButton(e) && (e.getButton() == MouseEvent.BUTTON1 ^ e.isMetaDown()))
                    || e.isPopupTrigger()) {
                JPopupMenu popup = new JPopupMenu("Metadata Viewer");
                popup.add(new JMenuItem("Do it!"));
                popup.show(this, pp.x, pp.y);
            }

        }
    */
}
