/*
 * File:     ImdiMDViewerPanel.java
 * Project:  MPI Linguistic Application
 * Date:     28 January 2009
 *
 * Copyright (C) 2001-2009  Max Planck Institute for Psycholinguistics
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package mpi.eudico.client.annotator.md.imdi;

import mpi.eudico.client.annotator.Constants;

import mpi.eudico.client.annotator.md.DefaultMDViewerComponent;
import mpi.eudico.client.annotator.md.spi.MDServiceProvider;


/**
 * Metadata viewer for IMDI metadata. Currently extends the
 * DefaultMDViewerComponent  by setting a custom table renderer for the
 * metadata keys. It might be necessary to have more flexibility in the
 * future. This viewer could then extend JPanel and implement
 * MDViewerComponent. A tree view could be offered as an alternative in that
 * case.
 *
 * @author Han Sloetjes
 * @version 1.0
 */
public class ImdiMDViewerPanel extends DefaultMDViewerComponent {
    /**
     * Creates a new ImdiMDViewerPanel instance
     */
    private ImdiMDViewerPanel() {
        super();
    }

    /**
     * Creates a new ImdiMDViewerPanel instance
     *
     * @param provider the metadata provider
     */
    public ImdiMDViewerPanel(MDServiceProvider provider) {
        super(provider);
    }

    /**
     * Initializes the components. Adds a custom renderer to the table.
     */
    protected void initComponents() {
        super.initComponents();
        mdTable.getColumn(keyColumn).setCellRenderer(new ImdiKeyRenderer());
        mdTable.setBackground(Constants.DEFAULTBACKGROUNDCOLOR);
    }
}
